import { ɵɵdefineInjectable, Injectable, Component, Input, NgModule } from '@angular/core';
import { BehaviorSubject, Subject, race, timer, of } from 'rxjs';
import { map, debounce, take, tap, switchMap } from 'rxjs/operators';
import { BrowserModule } from '@angular/platform-browser';

var SpinnerActionType;
(function (SpinnerActionType) {
    SpinnerActionType[SpinnerActionType["Show"] = 0] = "Show";
    SpinnerActionType[SpinnerActionType["Hide"] = 1] = "Hide";
    SpinnerActionType[SpinnerActionType["None"] = 2] = "None";
})(SpinnerActionType || (SpinnerActionType = {}));
;
class SpinnerAction {
}
class NgxLoadingSpinnerService {
    constructor() {
        this.visibleSubject = new BehaviorSubject(false);
        this.delayedSubject = new Subject();
        this.immediateSubject = new Subject();
        this.actionRequested = SpinnerActionType.None;
        this.actionExecuted = SpinnerActionType.None;
        this._timeout = 10000;
        this._threshold = 500;
        this.raceSubject();
    }
    set timeout(timeout) {
        this._timeout = timeout;
        this.patchShow();
    }
    set threshold(threshold) {
        this._threshold = threshold;
        this.patchShow();
    }
    raceSubject() {
        let delayedSubject = this.delayedSubject.pipe(map(t => ({ type: t, immediate: false })));
        let immediateSubject = this.immediateSubject.pipe(map(t => ({ type: t, immediate: true })));
        let thresholdObserver = race(delayedSubject.pipe(debounce(() => timer(this._threshold))), immediateSubject).pipe(take(1), tap((action) => {
            this.updatePublicState(action.type);
        }), map((action) => {
            if (action.type == SpinnerActionType.Show) {
                return { type: SpinnerActionType.Hide, immediate: false };
            }
            else {
                return action;
            }
        }));
        let timeoutObserver = race(thresholdObserver.pipe(switchMap((action) => (action.immediate ? of(action) : timer(this._timeout).pipe(map(_ => action))))), immediateSubject).pipe(take(1));
        timeoutObserver.subscribe((action) => {
            this.updatePublicState(action.type);
        }, error => console.error(error), () => this.raceSubject());
    }
    show() {
        this.actionRequested = SpinnerActionType.Show;
        this.immediateSubject.next(SpinnerActionType.None);
        this.delayedSubject.next(SpinnerActionType.Show);
    }
    hide() {
        this.actionRequested = SpinnerActionType.Hide;
        this.immediateSubject.next(SpinnerActionType.Hide);
    }
    //
    patchShow() {
        if (this.actionRequested == SpinnerActionType.Show) {
            if (this.actionExecuted == SpinnerActionType.Show) {
            }
            else {
                this.immediateSubject.next(SpinnerActionType.None);
                this.delayedSubject.next(SpinnerActionType.Show);
            }
        }
    }
    updatePublicState(type) {
        switch (type) {
            case SpinnerActionType.Show:
                this.actionExecuted = type;
                this.visibleSubject.next(true);
                break;
            case SpinnerActionType.Hide:
                this.actionExecuted = type;
                this.visibleSubject.next(false);
                break;
        }
    }
}
NgxLoadingSpinnerService.ɵprov = ɵɵdefineInjectable({ factory: function NgxLoadingSpinnerService_Factory() { return new NgxLoadingSpinnerService(); }, token: NgxLoadingSpinnerService, providedIn: "root" });
NgxLoadingSpinnerService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
NgxLoadingSpinnerService.ctorParameters = () => [];

class NgxLoadingSpinnerComponent {
    constructor(service) {
        this.service = service;
        this.zIndex = 100;
        this.visible = false;
    }
    set timeout(v) {
        this.service.timeout = v;
    }
    set threshold(v) {
        this.service.threshold = v;
    }
    ngOnInit() {
        this.subscribeToService();
    }
    subscribeToService() {
        this.service.visibleSubject.subscribe(spinnerVisible => {
            this.visible = spinnerVisible;
        });
    }
}
NgxLoadingSpinnerComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-loading-spinner',
                template: "<div class=\"overlay noselect\" [class.on]=\"visible\" [class.off]=\"!visible\" [style.zIndex]=\"zIndex\">\r\n    <div class=\"spinner-container\" *ngIf=\"!template; else showTemplate\">\r\n        <div class=\"spinner-item lds-default\"><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div></div>\r\n        <div class=\"spinner-item loading-text\" *ngIf=\"loadingText\">{{ loadingText }}</div>\r\n    </div>\r\n</div>\r\n\r\n<ng-template #showTemplate>\r\n  <div class=\"spinner-container\" [innerHTML]=\"template\"></div>\r\n</ng-template>\r\n",
                styles: [".noselect{-moz-user-select:none;-ms-user-select:none;-webkit-touch-callout:none;-webkit-user-select:none;user-select:none}.overlay{background-color:rgba(24,24,24,.5);height:100%;left:0;opacity:0;pointer-events:none;position:fixed;top:0;transition:opacity .5s;width:100%}.overlay.on{opacity:1;pointer-events:auto}.spinner-container{align-items:center;color:#fff;display:flex;flex-direction:column;font-family:Arial,Helvetica Neue,Helvetica,sans-serif;height:100%;justify-content:center}.loading-text{font-size:1.5em;text-align:center;text-shadow:0 0 .5rem #202020}.lds-default{display:inline-block;height:80px;position:relative;width:80px}.lds-default div{-webkit-animation:lds-default 1.2s linear infinite;animation:lds-default 1.2s linear infinite;background:#fff;border-radius:50%;height:6px;position:absolute;width:6px}.lds-default div:first-child{-webkit-animation-delay:0s;animation-delay:0s;left:66px;top:37px}.lds-default div:nth-child(2){-webkit-animation-delay:-.1s;animation-delay:-.1s;left:62px;top:22px}.lds-default div:nth-child(3){-webkit-animation-delay:-.2s;animation-delay:-.2s;left:52px;top:11px}.lds-default div:nth-child(4){-webkit-animation-delay:-.3s;animation-delay:-.3s;left:37px;top:7px}.lds-default div:nth-child(5){-webkit-animation-delay:-.4s;animation-delay:-.4s;left:22px;top:11px}.lds-default div:nth-child(6){-webkit-animation-delay:-.5s;animation-delay:-.5s;left:11px;top:22px}.lds-default div:nth-child(7){-webkit-animation-delay:-.6s;animation-delay:-.6s;left:7px;top:37px}.lds-default div:nth-child(8){-webkit-animation-delay:-.7s;animation-delay:-.7s;left:11px;top:52px}.lds-default div:nth-child(9){-webkit-animation-delay:-.8s;animation-delay:-.8s;left:22px;top:62px}.lds-default div:nth-child(10){-webkit-animation-delay:-.9s;animation-delay:-.9s;left:37px;top:66px}.lds-default div:nth-child(11){-webkit-animation-delay:-1s;animation-delay:-1s;left:52px;top:62px}.lds-default div:nth-child(12){-webkit-animation-delay:-1.1s;animation-delay:-1.1s;left:62px;top:52px}@-webkit-keyframes lds-default{0%,20%,80%,to{transform:scale(1)}50%{transform:scale(1.5)}}@keyframes lds-default{0%,20%,80%,to{transform:scale(1)}50%{transform:scale(1.5)}}"]
            },] }
];
NgxLoadingSpinnerComponent.ctorParameters = () => [
    { type: NgxLoadingSpinnerService }
];
NgxLoadingSpinnerComponent.propDecorators = {
    timeout: [{ type: Input }],
    threshold: [{ type: Input }],
    zIndex: [{ type: Input }],
    loadingText: [{ type: Input }],
    template: [{ type: Input }]
};

class NgxLoadingSpinnerModule {
    static forRoot() {
        return {
            ngModule: NgxLoadingSpinnerModule,
            providers: [NgxLoadingSpinnerService]
        };
    }
}
NgxLoadingSpinnerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgxLoadingSpinnerComponent],
                imports: [
                    BrowserModule
                ],
                exports: [NgxLoadingSpinnerComponent]
            },] }
];

/*
 * Public API Surface of ngx-loading-spinner
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxLoadingSpinnerComponent, NgxLoadingSpinnerModule, NgxLoadingSpinnerService };
//# sourceMappingURL=k-adam-ngx-loading-spinner.js.map
